from ninja import NinjaAPI, ModelSchema, Router
from ninja.security import django_auth
from django.contrib.auth.models import User

from config.api import router as config_router
from concierge.api import router as concierge_router

core_router = Router()

class UserSchema(ModelSchema):
  class Meta:
    model = User
    fields = ["id", "username", "email", "is_staff", "is_superuser", "first_name", "last_name"]

@core_router.get("/me", response=UserSchema)
def me(request):
  return request.user



api = NinjaAPI(
  title="Portier API",
  version="2.0.0",
  description="HTTP API for Portier. Use this to interact with the Portier backend.",
  csrf=False, # Disable CSRF for now
)

api.add_router("/", core_router, auth=django_auth, tags=["Core"])
api.add_router("/config/", config_router, auth=django_auth, tags=["Configuration"])
api.add_router("/concierge/", concierge_router, auth=None, tags=["Concierge"])
